*Recur.prg - a program that demonstrates recursive programming. 

*Written by Patrick Finucane

*In order to run this program, create a table called Files.DBF.
*The fields are Fname,C,12... Fsize,N,10... Fdate,D...
*Ftime,C,10...  and Fdir,C,65.

*This program will search for all (recommended is Ascii) files
*contained in the selected subdirectory and all other subordinate
*directories that meet the entered filespec.  You can also
*specify that the file contain a string.  For example, *.PRG as
*the file spec, and STR as the File Contains will find all PRGs
*with STR in any part of the file name; STR.PRG XXSTR.PRG, and
*ABCSTR1.PRG.

set talk off
release all
clear

cDir_select = getdir()		&& directory name to search 

if not empty(cDir_select)	&& run if a directory was selected

  set safety off
  use files exclusive
  zap
	
  *these are our global variables
  cDirspec  = "*."
  cFilespec = "*.*" + space(9)  && length of maximum file name
  cContain  = space(8)		&& does filename contain this?

  *get our search criteria
  @ 1,1 say "File spec:     " get cFilespec function "!"
  @ 2,1 say "File Contains: " get cContain  function "!"
  read

  *find the files 
  if not empty(cFilespec)
    clear
    ? "Searching..."
    do filefind with cDir_select
  endif	
	
  *view the results
  if reccount() # 0
    go top
    on key label ENTER do viewfile  &&view the file
    browse nomodify
    on key label ENTER				&&release ON KEY
  endif
		
  close data
	
endif	

return



procedure filefind
parameter cCurdir			&& directory to start from

private aDirname			&& array to hold directory name
private aFilename			&& array to hole file names
private nRowdir				&& number of directories in aDirname
private nRowfile			&& number of files in aFilename
private nFor				&& for-loop counter
private lDir				&& output the directory name?
private lFile				&& output the file name?

=adir(aDirname,cCurdir +  cDirspec,"D")
=adir(aFilename,cCurdir + cFilespec)

nRowdir  = iif(type("aDirname") # "U",alen(aDirname,1),0)
nRowfile = iif(type("aFilename") # "U",alen(aFilename,1),0)

lDir = .T.

for nFor = 1 to nRowfile
  if substr(aFilename(nFor,5),5,1) # "D"
    lFile = .T.
    if not empty(cContain)
      if "."$aFilename(nFor,1)
        cPrgname = substr(aFilename(nFor,1),1,;
        at(".",aFilename(nFor,1)) - 1)
      else
        cPrgname = aFilename(nFor,1)
      endif
	
      if not trim(cContain)$cPrgname
        lFile = .F.
      endif
  endif
		
  *output the directory name 
  if lDir and lFile
    ? cCurdir
    lDir = .F.
  endif
		
  *output the file name and append into database
  if lFile
    ? space(5) + aFilename(nFor,1)
    append blank
    replace Fname with aFilename(nFor,1), ;
      Fsize with aFilename(nFor,2), ;
      Fdate with aFilename(nFor,3), ;
      Ftime with aFilename(nFor,4), ;
      Fdir  with cCurdir
    endif
						
  endif
endfor

release aFilename	&&free up memory

for nFor = 1 to nRowdir

  if substr(aDirname(nFor,5),5,1) = "D" and ;
    substr(aDirname(nFor,1),1,1) # "."
      cNextdir = cCurdir + aDirname(nFor,1) + "\"
      do filefind with cNextdir
  endif	

endfor
return


procedure viewfile
cFilename = trim(Fdir) + Fname
modify command &cFilename noedit
return